@@ -2,20 +2,18 @@
 # Part of the hydroGOF R package, http://www.rforge.net/hydroGOF/ ; 
 #                                 http://cran.r-project.org/web/packages/hydroGOF/
 # Copyright 2011-2012 Mauricio Zambrano-Bigiarini
-# Distributed under GPL 2 or lat
+# Distributed under GPL 2 or later
 
-###########################################################################
-# 'ggof': Graphical comparision bewtween two vectors (numeric, ts or zoo),#
-#         with several numerical goodness of fit printed as a legend      #
-###########################################################################
-# External package required: 'zoo'                              #
-# External library required: 'lib_TSM_in_Hydrological_modelling #
-################################################################# 
-#  Started:  03 Mar 2009                #
-#  Updates:  Apr, May 2009              #
-#            2010                       #
-#            17-Apr-2011                # 
-#########################################     
+################################################################################
+# 'ggof': Graphical comparison between two vectors (numeric, ts, zoo, xts),    #
+#         with several numerical goodness-of-fit measures as a legend          #
+################################################################################
+#  Started:  03 Mar 2009                                                       #
+#  Updates:  Apr, May 2009                                                     #
+#            2010                                                              #
+#            17-Apr-2011                                                       # 
+#            15-Oct-2012                                                       #
+################################################################################     
                                           
       
 ggof <- function (sim, obs, 
@@ -56,17 +54,17 @@ ggof <- function (sim, obs,
                    
                    ...) {
 
-  # Checking that the user provied a valid argument for 'sim' &'obs'   
+  # Checking class 'sim' &'obs'   
   valid.class <- c("xts", "zoo", "numeric", "integer")    
   if (length(which(!is.na(match(class(sim), valid.class )))) <= 0)  
          stop("Invalid argument: 'class(sim)' must be in c('xts', 'zoo', 'numeric', 'integer')")
   if (length(which(!is.na(match(class(obs), valid.class )))) <= 0)
          stop("Invalid argument: 'class(obs)' must be in c('xts', 'zoo', 'numeric', 'integer')")
          
-  # Checking that the user provied the same length for 'sim' and 'obs'      
+  # Checking length
   if ( length(sim) != length(obs) )  
-     stop(paste("Invalid argument: 'obs' and 'sim' must have the same length ! (", 
-                length(obs), "vs", length(sim), ")"  ,sep=" ") )
+     stop("Invalid argument: 'obs' and 'sim' must have the same length ! (", 
+          length(obs), "vs", length(sim), ")")
                 
   # 'xname' and 'yname' values
   sim.name <- deparse(substitute(sim))
@@ -75,51 +73,59 @@ ggof <- function (sim, obs,
   # 'legend' value
   if (missing(legend)) legend <- c(sim.name, obs.name)
                    
-  # Checking that the user provied the same sampling frequency for 'sim' and 'obs',
-  # when 'sim' and 'obs' are 'zoo' objects      
+  # Checking same sampling frequency
   if ( zoo::is.zoo(obs) & zoo::is.zoo(sim)) {
-      if ( hydroTSM::sfreq(sim) != hydroTSM::sfreq(obs) ) {
-         stop(paste("Invalid argument: 'obs' and 'sim' have different sampling frequency ! (", 
-                   hydroTSM::sfreq(obs), "vs", hydroTSM::sfreq(sim), ")"  ,sep=" ") ) }
-  } # IF end
-         
-  # If the user didn't provide a title for the plot, the default is used 
-  if ( missing(main) ) main <- "Observations vs Simulations"         
+#      sim.freq <- xts::periodicity(sim)$scale
+#      obs.freq <- xts::periodicity(obs)$scale
+#      if ( sim.freq != obs.freq )
+#         stop("Invalid argument: 'obs' and 'sim' have different sampling frequency ! (", 
+#              obs.freq, " vs ", sim.freq, ")" )
+      if (all.equal(time(obs), time(sim)) != TRUE)
+        stop("Invalid argument: 'obs' and 'sim' have different time stamps !")
+  } # IF end    
           
   # If the user provided values 'for 'dates'
   if (!missing(dates)) {
   
     # Checking that 'dates' have the same length than 'sim' ( and 'obs')      
     if ( length(dates) != length(sim) )  
-         stop("Invalid argument: 'dates' and 'sim' must have the same length")
+        stop("Invalid argument: 'dates' and 'sim' must have the same length")
   
     # Checking that 'dates' have the right class
-    if (is.na(match(class(dates), c("character", "factor", "Date")))) 
-        stop("Invalid argument: 'class(dates)' must be in c('character', 'factor', 'Date')")
+    if (is.na(match(class(dates), c("character", "factor", "Date", "POSIXct")))) 
+        stop("Invalid argument: 'class(dates)' must be in c('character', 'factor', 'Date', 'POSIXct')")
         
     # If 'dates' is a factor or character, it have to be converted into 'Date' class, 
     # using the date format  specified by 'date.fmt'
-     if ( !is.na( match(class(dates), c("factor", "character") ) ) ) 
-        dates <- as.Date(dates, format= date.fmt)    
+     if ( class(dates)[1] %in% c("factor", "character") ) {
+        ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
+             grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
+             grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE),
+             subdaily <- TRUE, subdaily <- FALSE )
+        ifelse(subdaily, dates <- as.POSIXct(dates, format= date.fmt), 
+                         dates <- as.Date(dates, format= date.fmt) )  
+     } # IF end  
     
     # If 'obs' is 'zoo' and the user provides the dates (probably new dates)
     if ( zoo::is.zoo(obs) ) time(obs) <- dates
     # If 'sim' is 'zoo' and the user provides the dates  (probably new dates)
     if ( zoo::is.zoo(sim) ) time(sim) <- dates   
     
-  } else if (!zoo::is.zoo(obs)) message("[Note: You didn't provide dates, so only a numeric index will be used in the time axis.]")      
- 
+  } else if (!zoo::is.zoo(obs)) 
+            message("[ Note: You did not provide dates, so only a numeric index will be used in the time axis ]")    
   
   # If 'class(obs)' is not 'zoo' and the user provides the dates, then we turn it into a zoo class
   if ( !zoo::is.zoo(obs) & !missing(dates) ) { 
     obs <- hydroTSM::vector2zoo(x=obs, dates=dates, date.fmt=date.fmt)        
   } # If 'class(obs)' is 'zoo' and 'dates' are missing, dates are extracted from 'obs'
     else if ( zoo::is.zoo(obs) & missing(dates) ) {  
+      # class(time(x))== "POSIXct" for sub-daily time series
       # class(time(x))== "Date" for 'daily' and 'monthly' time series
       # class(time(x))== "character" for 'annual' time series
-      if ( class(time(obs)) == "Date" ) { dates <- time(obs) 
-      } else if ( class(time(obs)) == "character" ) {  
-             dates <- as.Date(time(obs), format="%Y") }      
+      if ( class(time(obs))[1] %in% c("Date", "POSIXct") ) { 
+         dates <- time(obs) 
+      } else if ( class(time(obs))[1] == "character" )  
+                 dates <- as.Date(time(obs), format="%Y")      
     } #ELSE END
   
   # If 'class(sim)' is not 'zoo' and the user provides the dates, then we turn it into a zoo class
@@ -127,10 +133,12 @@ ggof <- function (sim, obs,
     sim <- hydroTSM::vector2zoo(x=sim, dates=dates, date.fmt=date.fmt) 
   # If 'class(sim)' is 'zoo' and 'dates' are missing, dates are extracted from 'sim'
   } else if ( zoo::is.zoo(sim) & zoo::is.zoo(obs) & missing(dates) ) {
+      # class(time(x))== "POSIXct" for sub-daily time series
       # class(time(x))== "Date" for 'daily' and 'monthly' time series
       # class(time(x))== "character" for 'annual' time series
-      if ( class(time(sim)) == "Date" ) { dates <- time(obs) 
-      } else if ( class(time(sim)) == "character" ) {  
+      if ( class(time(sim))[1]  %in% c("Date", "POSIXct") ) { 
+         dates <- time(obs) 
+      } else if ( class(time(sim))[1] == "character" ) {  
              dates <- as.Date(time(sim), format="%Y") }
     } #ELSE END    
     
@@ -138,18 +146,22 @@ ggof <- function (sim, obs,
   if (is.na(match(ftype, c("o", "dm", "ma", "dma") ) ) ) 
       stop("Invalid argument: 'ftype' must be in c('o', 'dm', 'ma, 'dma')")
   
-  # If 'obs' and 'sim' are not zoo objetcs, the only possible value for 'ftype' is 'o'     
-  if ( !zoo::is.zoo(sim) & ! zoo::is.zoo(sim) ) {
+  # If 'obs' and 'sim' are not zoo objects, the only possible value for 'ftype' is 'o'     
+  if ( !zoo::is.zoo(sim) & !zoo::is.zoo(sim) ) {
      if (!is.na(match(ftype, c("dm", "ma", "dma") ) ) ) 
-      message("[Note: 'sim' & 'obs' are not zoo objects => 'ftype' was changed to 'o']")
+      message("[ Note: 'sim' & 'obs' are not zoo objects => 'ftype' was changed to 'o' ]")
       ftype <- "o"
-  } # IF end
+  } else if ( zoo::is.zoo(sim) ) 
+          sim.freq <- xts::periodicity(sim)$scale
          
   # Checking FUN, when 'ftype' involves monthly or annual values     
   if (!is.na(match(ftype, c("dm", "ma", "dma") ) ) & missing(FUN) ) 
          stop("Missing argument: 'FUN' must be provided when 'ftype' is in c('dm', 'ma, 'dma')")
+
+  # If the user did not provide a title for the plot, the default is used 
+  if ( missing(main) ) main <- "Observations vs Simulations"     
   
-  #Plotting acoording to the 'ftype' value:  
+  #Plotting according to the 'ftype' value:  
   if (ftype == "o") {
         
    # Drawing the original time series against time
@@ -166,17 +178,17 @@ ggof <- function (sim, obs,
          
   } else if (ftype=="dm") {
     
-      if (hydroTSM::sfreq(sim) != "daily") {      
+      if (sim.freq != "daily") {      
         stop("Invalid argument: 'sim' has to have a 'daily' sampling frequency")       
       } else {
-          # Generating a Monthly time series (Monthly mean of daily values):
+          # Generating a Monthly time series
           obs.monthly <- hydroTSM::daily2monthly(obs, FUN, na.rm)
           sim.monthly <- hydroTSM::daily2monthly(sim, FUN, na.rm)
           
           def.par <- par(no.readonly = TRUE) # save default, for resetting... 
           on.exit(par(def.par)) 
           
-          # If the user wants a legend, the screen is splitted into 2 rows and 2 colums, 
+          # If the user wants a legend, the screen is split into 2 rows and 2 columns, 
           # where the proportion of width of the 1st column to the 2nd one is 9:2
           if (gof.leg) {           
             layout( matrix( c(1,1,1,1,1,1,1,1,1,2,2,3,3,3,3,3,3,3,3,3,4,4), ncol=11, byrow=TRUE) ) 
@@ -199,7 +211,7 @@ ggof <- function (sim, obs,
                 legend=legend, leg.cex=leg.cex,
                 cal.ini=cal.ini, val.ini=val.ini, date.fmt=date.fmt, ... )
           
-          # It is necessay to set up the margins again, after the previous call to plot2
+          # It is necessary to set up the margins again, after the previous call to plot2
           par(mar=c(5, 4, 4, 0) + 0.1) # mar=c(bottom, left, top, right). Default values are: mar=c(5,4,4,2) + 0.1)           
           # Drawing the Monthly time series against time
           plot2(x=sim.monthly, y=obs.monthly, plot.type="single",
@@ -220,23 +232,23 @@ ggof <- function (sim, obs,
   
   else if (ftype=="ma") {
   
-    if  ( is.na( match( hydroTSM::sfreq(sim), c("daily", "monthly") ) ) ) {      
+    if  ( is.na( match( sim.freq, c("daily", "monthly") ) ) ) {      
       stop("Invalid argument: the sampling frequency of 'sim' has to be in c('daily', 'monthly'")       
     } else {
-        if ( hydroTSM::sfreq(sim) == "daily" ) {
-           # Generating a Monthly time series (Monthly mean of daily values):
+        if ( sim.freq == "daily" ) {
+           # Generating a Monthly time series 
            obs <- hydroTSM::daily2monthly(obs, FUN, na.rm)
            sim <- hydroTSM::daily2monthly(sim, FUN, na.rm)
         } # IF end
         
-        # Generating Annual time series (Annual mean of daily values)
+        # Generating Annual time series
         obs.annual <- hydroTSM::monthly2annual(obs, FUN, na.rm, out.fmt="%Y-%m-%d")
         sim.annual <- hydroTSM::monthly2annual(sim, FUN, na.rm, out.fmt="%Y-%m-%d")
         
         def.par <- par(no.readonly = TRUE) # save default, for resetting... 
         on.exit(par(def.par)) 
         
-        # If the user wants a legend, the screen is splitted into 2 rows and 2 colums, 
+        # If the user wants a legend, the screen is split into 2 rows and 2 columns, 
         # where the proportion of width of the 1st column to the 2nd one is 9:2
         if (gof.leg) {     
           layout( matrix( c(1,1,1,1,1,1,1,1,1,2,2,3,3,3,3,3,3,3,3,3,4,4), ncol=11, byrow=TRUE) )
@@ -258,7 +270,7 @@ ggof <- function (sim, obs,
               legend=legend, leg.cex=leg.cex,
               cal.ini=cal.ini, val.ini=val.ini, date.fmt=date.fmt, ... )
         
-        # It is necessay to set up the margins again, after the previous call to plot2
+        # It is necessary to set up the margins again, after the previous call to plot2
         par(mar=c(5, 4, 4, 0) + 0.1)                
         # Drawing the Annual time series against time
         plot2(x=sim.annual, y=obs.annual, plot.type="single",
@@ -277,22 +289,22 @@ ggof <- function (sim, obs,
   
   else if (ftype=="dma") {
         
-    if (hydroTSM::sfreq(sim) != "daily") {      
-      stop("Invalid argument: the 'sim' has to have a 'Daily' sampling frequency")  
+    if (sim.freq != "daily") {      
+      stop("Invalid argument: 'sim' has to have a 'daily' sampling frequency")  
            
     } else {       
-          # Generating Monthly time series (Monthly mean of daily values):
+          # Generating Monthly time series 
           obs.monthly <- hydroTSM::daily2monthly(obs, FUN, na.rm)
           sim.monthly <- hydroTSM::daily2monthly(sim, FUN, na.rm)
           
-          # Generating Annual time series (Annual mean of daily values)
+          # Generating Annual time series 
           obs.annual <- hydroTSM::daily2annual(obs, FUN, na.rm, out.fmt = "%Y-%m-%d")
           sim.annual <- hydroTSM::daily2annual(sim, FUN, na.rm, out.fmt = "%Y-%m-%d")
           
           def.par <- par(no.readonly = TRUE) # save default, for resetting... 
           on.exit(par(def.par)) 
           
-          # If the user wants a legend, the screen is splitted into 2 rows and 2 colums, 
+          # If the user wants a legend, the screen is split into 2 rows and 2 columns, 
           # where the proportion of width of the 1st column to the 2nd one is 9:2
           if (gof.leg) {   
             # Setting up a screen with 3 rows and 2 columns 
@@ -315,7 +327,7 @@ ggof <- function (sim, obs,
                 legend=legend, leg.cex=leg.cex,
                 cal.ini=cal.ini, val.ini=val.ini, date.fmt=date.fmt, ... )
           
-          # It is necessay to set up the margins again, after the previous call to plot2
+          # It is necessary to set up the margins again, after the previous call to plot2
           par(mar=c(5, 4, 4, 0) + 0.1) # mar=c(bottom, left, top, right). Default values are: mar=c(5,4,4,2) + 0.1)        
           # Drawing the Monthly time series against time
           plot2(x=sim.monthly, y=obs.monthly, plot.type="single",  
@@ -329,7 +341,7 @@ ggof <- function (sim, obs,
                 legend=legend, leg.cex=leg.cex,
                 cal.ini=cal.ini, val.ini=val.ini, date.fmt=date.fmt, ... )
            
-          # It is necessay to set up the margins again, after the previous call to plot2
+          # It is necessary to set up the margins again, after the previous call to plot2
           par(mar=c(5, 4, 4, 0) + 0.1) # mar=c(bottom, left, top, right). Default values are: mar=c(5,4,4,2) + 0.1)        
           # Drawing the Annual time series against time
           plot2(x=sim.annual, y=obs.annual, plot.type="single",
