@@ -27,6 +27,12 @@ mae.default <- function (sim, obs, na.rm=TRUE, ...){
   
 mae.matrix <- function (sim, obs, na.rm=TRUE, ...){
 
+  # Checking that 'sim' and 'obs' have the same dimensions
+  if ( all.equal(dim(sim), dim(obs)) != TRUE )
+    stop( paste("Invalid argument: dim(sim) != dim(obs) ( [", 
+          paste(dim(sim), collapse=" "), "] != [", 
+          paste(dim(obs), collapse=" "), "] )", sep="") )
+
   mae <- colMeans( abs(sim - obs), na.rm= na.rm)  
                  
   return(mae)
